+++
tags = [ "games", "programming" ]
date = "2017-11-19T22:00:00+08:00"
title = "Audventure"
slug = "audventure"
+++

Sometime around 2013 I wrote a clone of the GBA game [bit Generations
SoundVoyager](https://www.nintendo.co.jp/n08/bit_g/) called
[audventure](https://audventure.pleasantprogrammer.com). SoundVoyager is
actually a collection of mini-games where sound is the main focus. You can
actually play the game blind, and at some point, that's pretty much what
happens.

## sound catcher

The signature mini-game in SoundVoyager is sound catcher. In the mini-game, you
can only move left and right at the bottom of the stage, while a "sound" falls
from the top. Your goal is to catch the sound which is signified by a green dot.
When you catch it, the sound or beat becomes part of the BGM and a new dot
appears with a different sound.

You can of course use your eyes and move accordingly, but if you put on
earphones, you can actually hear where the dot is, either on your left or right,
with it getting louder as it gets close to you. As you collect more sounds, the
dot gets more and more transparent. Eventually (and this is where it gets fun),
you won't be able to see the sounds anymore and will have to rely mostly on your
ears.

You can see what the original game looks like in [this
video](https://www.youtube.com/watch?v=C12WRgfIOC8) or you can play it under
*sound safari* in [audventure](https://audventure.pleasantprogrammer.com).

## WebAudio vs Flash

At the time I wrote audventure, only Chrome supported WebAudio. Also, the API
looked (and still looks) quite complicated. Flash on the other hand, was
starting to die, but still well-supported so I went with that. For the most
part, it worked okay though Chrome actually had timing issues when playing
sounds. Now, it doesn't work in any browser. I tried to debug the issues but
ultimately ended up just rewriting it to use WebAudio instead.

For the game, I needed to simulate the source of the sound in 2D/3D space. Flash
only really gives you stereo panning and volume control. With some maths, we can
actually get an acceptable solution. Less importantly, I needed to be able to
get frequency data of the currently playing "sound" to pulse the background. For
this, I actually had to implement the feature in the Flash library I was using.

With WebAudio, spatial audio is already built-in and you can simply give it the
coordinates of the sounds and the listener. There are some other options to
tweak, but for the most part, no complex math is needed. Getting frequency data
for a sound is also actually built-in and didn't take too long to integrate.

Overall, I was impressed by how much you can do with WebAudio out-of-the-box. I
kind of understand why it's complicated, but there's some simple functionality
that I wish was included. For example, there is no API to pause and then resume
playing an audio buffer. You have to manually save the elapsed time and play
from there.

## Other mini-games

So far I've only actually implemented the sound catcher mini-game. There are
around 4 different categories with slight variations in between.

### sound catcher / sound slalom

I've explained sound catcher a while ago; sound slalom is a minor variation on
that. Instead of waiting for the "sound" to reach you, you now have to guide
yourself in between 2 "poles" of sound, as in [slalom
skiing](https://en.wikipedia.org/wiki/Slalom_skiing). But this time, you can
also accelerate forward. The goal is to finish the course before the time runs
out.

### sound drive / sound chase

In sound drive, you're driving against the flow on a 5 lane road. You have to
avoid oncoming cars, trucks and animals until you reach the end. You're allowed
to change lanes and accelerate, and the game tracks your best time. Sound chase
is pretty much the same, except you're trying to catch up to a "sound".

### sound cannon

In sound cannon, you're immobile but can rotate within a 180 degree angle. Your
goal is too shoot down "sounds" which are heading your way. If a sound reaches
you, it's game over. You win when you kill all the sounds.

### sound picker / sound cock

In sound picker, you can move in a giant square field where various sounds are
scattered around. Your goal is to pick up all the sounds within the time limit.
Sound cock is similar, except the sounds are chickens and you have to chase them
around.

## Source Code

If you want to see the source code, you can check it out
[here](https://git.pleasantprogrammer.com/games/audventure). The sound files
aren't in the repo though, since I'm not quite sure about the licensing. If you
want to contribute music or sound effects, I'd gladly appreciate it.
